/**************************************************************
	LCD Serial communication monitor for PIC16F1827
	( 16F648A old hardware solution compatible)
	ACM0802C  8 colimns 2 lines
	SC1602BS 16 columns 2 lines (Default)
	SC242A	 24 columns 2 lines
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	Tools:
	MPLAB X IDE v6.20
	MPLAB   XC8 v2.50
---------------------------------------------------------------
Date	:Time	Version	Build	Note
---------------------------------------------------------------
Date	 Time	Version	Build	Note
20111011 23:00	1.00	---		begin the program
20241030 23:00  2.00            Corresponding new MPLABX IDE and C
---------------------------------------------------------------
	Pin assignment
	 1:RA2:O:LCD D6
	 2:RA3:O:LCD D7
	 3:RA4:0:(N.C)
	 4:RA5/MCLR:I:PICKIT2-1(W/10kPU)
	 5:GND
	 6:RB0:I:nMODE SWITCH
	 7:RB1:I:RX
	 8:RB2:O:TX(NOT USED)
	 9:RB3:O:PWM OUTPUT 83.8KHz
	 
	18:RA1:O:LCD D5
	17:RA0:O:LCD D4
	16:CLKI/NC:8MHz RESONATOR if selected External clock use
	15:CLKO/NC:8MHz RESONATOR if selected External clock use
	14:VDD:Power 2.7 TO 5V
	13:RB7:O:LCD E
	12:RB6:O:LCD RS
	11:RB5:I:nRIGHT SWITCH
	10:RB4:I:nLEFT SWITCH

---------------------------------------------------------------
	Display cursor 
	ASCII mode : cursor is blinking (always can switch with command Ctrl-Q/W)
	Binary mode: cursor is stacking (under ber)

	Behavior when code is received
	ascii code	12 		(ctrl +l)	LCD clear
	ascii code	0x0b	(ctrl +k)	Toggle line
	ascii code	0x0d	(Enter)		Return command
	ascii code	0x11	(ctrl +q)	Cursor off
	ascii code	0x17	(ctrl +w)	Cursor on
	ascii code	0x20 to 0xff		Display the corresponding ascii code
---------------------------------------------------------------
Be careful!
	This use LCD character module(SC242A(24 x 2 lines)) is different to power connection of SC1602 series.
	Please careful that the direction of module power polar. 
	
**************************************************************/

#include <xc.h>
#include "main.h"
#include "sci.h"
#include "lcd.h"
#include "eeprom.h"
#include "promdata.h"

//CONFIG1
#if defined (_InternalClock)
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

#else
#pragma config FOSC = HS        // Oscillator Selection (HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

#endif

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config DEBUG = OFF      // In-Circuit Debugger Mode (In-Circuit Debugger disabled, ICSPCLK and ICSPDAT are general purpose I/O pins)
#pragma config LVP = ON         // Low-Voltage Programming Enable (Low-voltage programming enabled)
                                // d݂WEnableɂ

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>

//interrupt handle
void __interrupt() my_isr_routine(void) { //Change to new notation ***2.00
	uint8_t temp;
	uint8_t data;
	//Handle RS232 Recieve interrupt
	if (PIR1bits.RCIF == 1) {
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		PIR1bits.RCIF = 0; //clear RS232 Recieve interrupt bit
		if (FIFO_datasize < MAX_BUFF) {
			Buffer_FIFO[FIFO_write_pointer] = data;
			FIFO_write_pointer = (uint8_t) ((FIFO_write_pointer + 1) & (MAX_BUFF - 1)); //ring buff 
			FIFO_datasize++;
		}
	}
	if( INTCONbits.TMR0IF==1 ){	//timer interval interrupt
		if (!SW_LEFT){
			while(!SW_LEFT);// button release check
			asm("RESET");//reset
		}
	
		if (!SW_RIGHT)	{
			temp = E2promRead(DisplayMode);
			if (temp != ASCIImode ) E2promWrite(DisplayMode,ASCIImode);
				else E2promWrite(DisplayMode,HEXmode);
			while(!SW_RIGHT);// button release check
			asm("RESET");//reset
		}

		if (sci_CheckOERR()) { //sci error
			RCSTAbits.CREN=1;
		}

	//timer interval
	TMR0IF =0 ;
	}
}

uint8_t Buffer_readdata(void){
	uint8_t wdat;
	//read FIFO buffer
	while(FIFO_datasize==0);	// pause if buffer is empty
	wdat=Buffer_FIFO[FIFO_read_pointer];
	FIFO_read_pointer=(FIFO_read_pointer+1)&(MAX_BUFF-1);//ring buff
	FIFO_datasize--;
	return wdat;//get wdat
//	return FIFO_datasize;//debug
}	

void Buffer_check(void){
	uint8_t data;
	if (FIFO_datasize==MAX_BUFF-1) {
		lcd_putch(0xFF);
		lcd_puts("Full");
		lcd_putch(0xFF);
		PIE1bits.RCIE = 0;	//terminal interrupt disable
		FIFO_datasize 		=0;	//FIFO clear
		FIFO_write_pointer 	=0;
		FIFO_read_pointer 	=0;
		RCSTAbits.CREN =0;	//Receive was aborted
		while (PIR1bits.RCIF ==1){
			data = sci_GetByte();		//Dummy read
		}
		delay_10ms(20);//200ms
		RCSTAbits.CREN = 1;	//Receive is start
		PIE1bits.RCIE = 1;	//terminal interrupt enable				
	}	
}

//Generate CG characters //***2.00
void lcd_CG_init(void) {
	uint8_t temp, i, j, CGRAM_Address;
	CGRAM_Address = CGRAM_OffsetAddress;
	lcd_write((uint8_t)(0x40 + CGRAM_Address)); //set CGRAM mode
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 8; j++) {
			temp = E2promRead(CGRAM_Address++);
			lcd_cgwrite(temp);
		}
	}
	lcd_write(0x80); //DD RAM mode 
}


void delay_10ms(uint8_t temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}


//------------------main
void main(void)
{
// Register
	unsigned int speed = 38400; //default communication speed set
	uint8_t temp,data,buffspeed;
//	Port initalize
//		     76543210
	LATA = 0b11100000;	//Port Initalize
	LATB = 0b00000000;	//Port Initalize

#if defined (_InternalClock)
	TRISA = 0b00110000; //if internal system clock use
#else
	TRISA = 0b11110000; //if external system clock use
#endif

	TRISB = 0b00110111; //
	ANSELA	= 0x00;		//All port are use for digital port
	ANSELB	= 0x00;		//All port are use for digital port
	CM1CON0 = 0x07;		//PORTA is all digital I/O
	APFCON0bits.RXDTSEL	= 0;		//RB1 pin assignment as RXD (16F88 compatible)
	OSCCON	= 0b01110010;	// PLL off,8MHz,IntOSC
	OPTION_REGbits.nWPUEN  = 0;		//PORTB weak pullup


//Init
	OPTION_REG = 0b00000111; // prescale 1/256 RBPU= EN
	INTCONbits.GIE  = 0;//Int off

//timer0 setting (interrupt event timer 3.328ms)
	INTCONbits.TMR0IE = 0;//timer interrupt is off
	TMR0 = 0;//interrupt timer counter set

//PWM initialize setting (83.8KHz on RB3 @8MHz)

	CCP1CON	=	0b00001100;	//Single output,LSBset,PWM mode: P1A, P1C active-high; P1B, P1D active-high
	PR2 = 0x17; // resolution is 6.5bit mode
	CCPR1L = 0x0c; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1

/*
//PWM test (31.25KHz on RB3 @8MHz) test only
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0xff; // resolution is 8bit mode
	CCPR1L = 0x20; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
*/

//LCD init
	delay_10ms(20);	//stabilize time
	lcd_init();		//lcd initalize
    lcd_CG_init();  //lcd CG RAM set
	lcd_clear();	//lcd clear
//setting mode
	if (!SW_MODE ){
		lcd_write(LCD_CursorOff);
		lcd_goto(LCD_LineNumber1);
		lcd_puts(VersionInfo);	//version kind of LCD
		lcd_puts(VersionCode);	//version code
		while(!SW_MODE);
		lcd_clear();
		lcd_goto(LCD_LineNumber2);
#ifndef	_LCD0802
		lcd_puts("Set  DOWN  UP");
#else
		lcd_puts("Set -  +");
#endif	

		buffspeed = E2promRead(COM_Speed);
		while(1){
			delay_10ms(5);		//50ms;
			lcd_goto(LCD_LineNumber1);
			switch (buffspeed){
				case Speed1200:
					lcd_puts(" 120");
					break;
				case Speed2400:
					lcd_puts(" 240");
					break;
				case Speed4800:
					lcd_puts(" 480");
					break;
				case Speed9600:
					lcd_puts(" 960");
					break;
				case Speed19200:
					lcd_puts("1920");
					break;
				case Speed31250:
					lcd_puts("3125");
					break;
				case Speed38400:
					lcd_puts("3840");
					break;
				default: // data is broken
					buffspeed =Speed38400;
			}

			lcd_puts("0bps");
		
			//key check
			if (!SW_LEFT ) {
				if ( buffspeed > MIN_Speed) buffspeed--;
				while(!SW_LEFT);

			}
			if (!SW_RIGHT  ){
				if ( buffspeed < MAX_Speed) buffspeed++;
				while(!SW_RIGHT);

			}
			if (!SW_MODE  ){
				if (buffspeed != E2promRead(COM_Speed)){
					E2promWrite(COM_Speed,buffspeed);
					lcd_goto(LCD_LineNumber2);
					lcd_puts("OK ");
					lcd_write(LCD_CursorBlink);
					delay_10ms(50);	//500ms	
				}
				while(!SW_MODE);
				break;
			}
		}
	}

	buffspeed = E2promRead(COM_Speed);
//sci initialize
	switch (buffspeed){
		case Speed1200:
			speed = 1200;
			break;
		case Speed2400:
			speed = 2400;
			break;
		case Speed4800:
			speed = 4800;
			break;
		case Speed9600:
			speed = 9600;
			break;
		case Speed19200:
			speed = 19200;
			break;
		case Speed31250:
			speed = 31250;
			break;
		case Speed38400: //This speed is not accept HEX mode.
			speed = 38400;
			break;
		default:
			speed = 38400;
	}

	sci_Init(speed ,SCI_EIGHT);// Baud set and Bit set

//interrupt enable
//	INTEDG = 0; //RB0 Key interrupt fall edge
//	INTE = 1;//RB0 is interrupt mode
	INTCONbits.TMR0IE = 1;//timer interrupt on
	PIE1bits.RCIE = 1;//recive interrupt on
	INTCONbits.PEIE = 1;//Peripheral interrupt Enable 
	INTCONbits.GIE  = 1;//Global interrupt Enable 

//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TXSTAbits.TRMT);
		RCSTAbits.CREN=1;
	}

//Main routiene
	lcd_init();
//    lcd_CG_init(); //***2.00
	lcd_clear();	//lcd clear

/*
//test
	temp = 0x20;
	while(1){
		if ( temp == 0xff ) temp = 0x20;
		lcd_putch(temp++);
		delay_10ms(20);
	}
	//test end	
*/

	temp = E2promRead(DisplayMode);

	switch (temp){
		case HEXmode :
			lcd_write(LCD_CursorOn);	//Init cursor
			//ASCII to HEX convert(Use less or equal 19200bps)
//			lcd_puts("HEX:");
//			lcd_goto(LCD_LineNumber2);
			while(1)	{
				data= Buffer_readdata();//Buffer read 
			    //Send high nibble
			    if( (data>>4) > 9 )
			        lcd_putch( 'A' - 10 + (data>>4) );
			    	else lcd_putch( '0' + (data>>4) );
			    //Send low nibble
			    if( (data&0x0F) > 9 )
			        lcd_putch( 'A' - 10 + (data&0x0F) );
			    	else lcd_putch( '0' + (data&0x0F) );

				Buffer_check();	//Buffer check
			}
			break;	

		default:
			lcd_write(LCD_CursorBlink);	//Init cursor
//			lcd_puts("ASCII:");
//			lcd_goto(LCD_LineNumber2);
			while(1){
				data= Buffer_readdata();//Buffer read 
				switch ( data ){
					case 12	:	//clear
						lcd_clear();
						break;
					case 0x0b	:	//Verticcal tab
						lcd_newline(1);
						break;
					case 0x0d	:	//return command
						lcd_newline(0);
						break;
					case 0x11	:	//cursor off(ctrl +q)
						lcd_write(LCD_CursorOff);//cursor off
						break;
					case 0x17	:	//cursor on(ctrl +w)
						lcd_write(LCD_CursorBlink);//cursor on
						break;
					default		:
						//CG RAM display progress ***1.52
						if (data >= 0x20) {
							if ((data > 0x7F) && (data < 0x90)) {
								lcd_putch((uint8_t) (data - 0x80));
							} else {
								lcd_putch(data);
							}
						}
						break;
				}
			}
		break;

	}

}
